@@ -1,18 +1,22 @@
 //! Completion of paths, i.e. `some::prefix::$0`.
 
+use std::iter;
+
 use hir::HasVisibility;
 use rustc_hash::FxHashSet;
-use syntax::AstNode;
+use syntax::{ast, AstNode};
 
-use crate::{CompletionContext, Completions};
+use crate::{context::PathCompletionContext, CompletionContext, Completions};
 
 pub(crate) fn complete_qualified_path(acc: &mut Completions, ctx: &CompletionContext) {
     if ctx.is_path_disallowed() {
         return;
     }
-    let path = match ctx.path_qual() {
-        Some(path) => path,
-        None => return,
+    let (path, use_tree_parent) = match &ctx.path_context {
+        Some(PathCompletionContext { qualifier: Some(qualifier), use_tree_parent, .. }) => {
+            (qualifier, *use_tree_parent)
+        }
+        _ => return,
     };
 
     let resolution = match ctx.sema.resolve_path(path) {
@@ -39,6 +43,23 @@ pub(crate) fn complete_qualified_path(acc: &mut Completions, ctx: &CompletionCon
         return;
     }
 
+    if ctx.in_use_tree() {
+        if iter::successors(Some(path.clone()), |p| p.qualifier())
+            .all(|p| p.segment().and_then(|s| s.super_token()).is_some())
+        {
+            acc.add_keyword(ctx, "super::");
+        }
+        // only show `self` in a new use-tree when the qualifier doesn't end in self
+        if use_tree_parent
+            && !matches!(
+                path.segment().and_then(|it| it.kind()),
+                Some(ast::PathSegmentKind::SelfKw)
+            )
+        {
+            acc.add_keyword(ctx, "self");
+        }
+    }
+
     // Add associated types on type parameters and `Self`.
     resolution.assoc_type_shorthand_candidates(ctx.db, |_, alias| {
         acc.add_type_alias(ctx, alias);
@@ -333,7 +354,7 @@ trait Trait { fn m(); }
 fn foo() { let _ = Trait::$0 }
 "#,
             expect![[r#"
-                fn m() fn()
+                fn m() (as Trait) fn()
             "#]],
         );
     }
@@ -350,7 +371,7 @@ impl Trait for S {}
 fn foo() { let _ = S::$0 }
 "#,
             expect![[r#"
-                fn m() fn()
+                fn m() (as Trait) fn()
             "#]],
         );
     }
@@ -367,7 +388,7 @@ impl Trait for S {}
 fn foo() { let _ = <S as Trait>::$0 }
 "#,
             expect![[r#"
-                fn m() fn()
+                fn m() (as Trait) fn()
             "#]],
         );
     }
@@ -393,14 +414,14 @@ trait Sub: Super {
 fn foo<T: Sub>() { T::$0 }
 "#,
             expect![[r#"
-                ta SubTy        type SubTy;
-                ta Ty           type Ty;
-                ct C2           const C2: ();
-                fn subfunc()    fn()
-                me submethod(…) fn(&self)
-                ct CONST        const CONST: u8;
-                fn func()       fn()
-                me method(…)    fn(&self)
+                ta SubTy (as Sub)        type SubTy;
+                ta Ty (as Super)         type Ty;
+                ct C2 (as Sub)           const C2: ();
+                fn subfunc() (as Sub)    fn()
+                me submethod(…) (as Sub) fn(&self)
+                ct CONST (as Super)      const CONST: u8;
+                fn func() (as Super)     fn()
+                me method(…) (as Super)  fn(&self)
             "#]],
         );
     }
@@ -433,14 +454,14 @@ impl<T> Sub for Wrap<T> {
 }
 "#,
             expect![[r#"
-                ta SubTy        type SubTy;
-                ta Ty           type Ty;
-                ct CONST        const CONST: u8 = 0;
-                fn func()       fn()
-                me method(…)    fn(&self)
-                ct C2           const C2: () = ();
-                fn subfunc()    fn()
-                me submethod(…) fn(&self)
+                ta SubTy (as Sub)        type SubTy;
+                ta Ty (as Super)         type Ty;
+                ct CONST (as Super)      const CONST: u8 = 0;
+                fn func() (as Super)     fn()
+                me method(…) (as Super)  fn(&self)
+                ct C2 (as Sub)           const C2: () = ();
+                fn subfunc() (as Sub)    fn()
+                me submethod(…) (as Sub) fn(&self)
             "#]],
         );
     }
