@@ -295,6 +295,7 @@ impl CompletionItem {
             label,
             insert_text: None,
             is_snippet: false,
+            trait_name: None,
             detail: None,
             documentation: None,
             lookup: None,
@@ -398,6 +399,7 @@ pub(crate) struct Builder {
     source_range: TextRange,
     completion_kind: CompletionKind,
     import_to_add: Option<ImportEdit>,
+    trait_name: Option<String>,
     label: String,
     insert_text: Option<String>,
     is_snippet: bool,
@@ -432,8 +434,11 @@ impl Builder {
             if original_path_label.ends_with(&label) {
                 label = original_path_label;
             } else {
-                format_to!(label, " ({})", original_path)
+                format_to!(label, " (use {})", original_path)
             }
+        } else if let Some(trait_name) = self.trait_name {
+            insert_text = insert_text.or_else(|| Some(label.clone()));
+            format_to!(label, " (as {})", trait_name)
         }
 
         let text_edit = match self.text_edit {
@@ -468,6 +473,10 @@ impl Builder {
         self.label = label.into();
         self
     }
+    pub(crate) fn trait_name(&mut self, trait_name: impl Into<String>) -> &mut Builder {
+        self.trait_name = Some(trait_name.into());
+        self
+    }
     pub(crate) fn insert_text(&mut self, insert_text: impl Into<String>) -> &mut Builder {
         self.insert_text = Some(insert_text.into());
         self
